############################################
#SSA decomposition and grouping diagnostics#
############################################

#######################################################
#Contact: R. Huffaker                                 #   
#Professor                                            #
#Department of Agricultural and Biological Engineering#
#Institute of Food and Agricultural Sciences          #
#University of Florida                                #
#rhuffaker@ufl.edu                                    #
#######################################################

#Remove objects in memory
rm(list=ls(all=TRUE))

################################
#Load Observed Times Series 'x'#
################################
data<-read.csv(".csv")
x<-data$
x<-na.omit(x) #needed if columns in spreadsheet not same length
x<-as.vector(x)

###################
#SSA Decomposition#
###################
#Package 'Rssa': Authors Anton Korobeynikov, Alex Shlemov, 
  #Konstantin Usevich, Nina Golyandina
library(Rssa)
L=length(x)/2 #input SSA window length L
s<-ssa(x,L,kind="toeplitz-ssa") 

########################################  
#Run diagnostics to group eigentriplets#
########################################
#Eigenspectrum: Plots square roots of eigenvalues ('singular values') 
 #in each eigentriplet from largest to smallest in magnitude. There are 
 #L singular values. When the data are mean-removed and the Toeplitz 
 #version of SSA is applied, eigenvalues measure the portion of variance 
 #in the observed time series from its mean explained by the associated 
 #eigentriplets ('partial variance'). Singular values along the steeper 
 #portion are associated with eigentriplets potentially forming the basis 
 #of the deterministic signal, while those on the flat portion ('noise 
 #floor') are associated with 'white noise'. The largest singular value 
 #is associated with slow-moving trend. Pairs of singular values that are 
 #equal (creating steps in the eigenspectrum) are associated with 
 #eigentriplets forming the basis of potential oscillations.
eigenvalues<-plot(s,numvalues=20)

#Eigenvectors: Plot eigenvectors in each eigentriplet to visually identify 
 #oscillations captured by eigentriplets. Eigenvectors are time series 
 #capturing oscillatory characteristics of observed time series data.
eigenvectors<-plot(s,type="vectors",idx=1:20)  

#Scatterplots: Pairwise scatterplots of eigenvector pairs as another 
 #visualization device. Scatterplots of sin and cos function with the same 
 #wavelength produce distinct geometric shapes that correspond to 
 #scatterplots of eigenvectors assocated with pairs of eigentriplets with 
 #identical singular values.
scatterplot<-plot(s,type="paired",idx=1:10,plot.contrib=FALSE)  

#Weighted correlation matrix: Check whether there is statistical 
 #independece between grouped eigentriplets
w.corr.plot<-plot(w<-wcor(s))
#Restrict plot to first 20 ordered eigentriplets
w.corr.res<-wcor(s,groups=c(1:20))  #table
w.corr.res.plot<-plot(w.corr.res)  #plot