####################
#SSA reconstruction#             
####################

#######################################################
#Contact: R. Huffaker                                 #   
#Professor                                            #
#Department of Agricultural and Biological Engineering#
#Institute of Food and Agricultural Sciences          #
#University of Florida                                #
#rhuffaker@ufl.edu                                    #
#######################################################

rm(list=ls(all=TRUE)) #Remove objects in memory

################################
#Load Observed Times Series 'x'#
################################
data<-read.csv(".csv")
x<-data$
x<-na.omit(x) #needed if columns in spreadsheet not same length
x<-as.vector(x)

##################################################################
#Eigentriplet groups:  Note groupings may be different for       # 
#same cycles in Caterpillar and Rssa. Note also that the command # 
#'parestimate' only works for groups with eigentriplet pairs and #
#will fail for groups that are not well paired.                  #
##################################################################
n.groups<-5  #number of eigentriplet groups
group.1<-c(1:5) #input trend--can be 1, 2, or more eigentriplets
group.2<-c(6,7) #input oscillations (only eigentriplet pairs)
group.3<-c(8,9) 
group.4<-c(10,11)
group.5<-c(13,14)

######################################
#Run singular spectrun analysis (ssa)#
######################################
#Package 'Rssa': Authors Anton Korobeynikov, Alex Shlemov, 
  #Konstantin Usevich, Nina Golyandina
library(Rssa)
L<-length(x)/2   #ssa window
s<-ssa(x,L,kind="toeplitz")  

setwd(dir.results)
##################################
#Results for 1 eigentriplet group#
##################################
if (n.groups==1) {
#Compute periods of eigentriplet groups
if(length(group.1)==2) {
  cycle.1<-parestimate(s,groups=list(group.1))[2]
  cycles<-cycle.1} 

#Reconstruct time series' from eigentriplet groups
r.1<-reconstruct(s,groups=group.1)  #group 1 
recon.1<-r.1$F1 
  
#Signal/Noise
#recon<-reconstruct(s,groups=list(1:5));recon<-r$F1;res<-residuals(r)
signal<-recon.1
noise<-x-signal  #residuals to composite reconstruction

#Export reconstructed series
data<-cbind(x,signal,noise,recon.1)
write.table(data,sep=",",col.names=c("data","signal","noise","recon.1"),
  "data.csv",row.names=FALSE)
} #end if n.group=1

###################################
#Results for 2 eigentriplet groups#
###################################
if (n.groups==2) {
#Compute periods of eigentriplet groups
if(length(group.1)==2) {
  cycle.1<-parestimate(s,groups=list(group.1))[2]}
if(length(group.2)==2){
  cycle.2<-parestimate(s,groups=list(group.2))[2]}
if(length(group.1)==2) {
  cycles<-cbind(cycle.1,cycle.2)} else {
  cycles<-cbind(cycle.2)}

#Reconstruct time series' from eigentriplet groups
r.1<-reconstruct(s,groups=group.1)  #group 1 
recon.1<-r.1$F1 
r.2<-reconstruct(s,groups=group.2)  #group 2 
recon.2<-r.2$F1 
  
#Signal/Noise
#recon<-reconstruct(s,groups=list(1:5));recon<-r$F1;res<-residuals(r)
signal<-recon.1+recon.2
noise<-x-signal  #residuals to composite reconstruction

#Export reconstructed series
data<-cbind(x,signal,noise,recon.1,recon.2)
write.table(data,sep=",",col.names=c("data","signal","noise","recon.1",
  "recon.2"),"data.csv",row.names=FALSE)
} #end if n.group=2

###################################
#Results for 3 eigentriplet groups#
###################################
if (n.groups==3) {
#Compute periods of eigentriplet groups
if(length(group.1)==2) {
  cycle.1<-parestimate(s,groups=list(group.1))[2]}
if(length(group.2)==2){
  cycle.2<-parestimate(s,groups=list(group.2))[2]}
if(length(group.3)==2) {
  cycle.3<-parestimate(s,groups=list(group.3))[2]}
if(length(group.1)==2) {
  cycles<-cbind(cycle.1,cycle.2,cycle.3)} else {
  cycles<-cbind(cycle.2,cycle.3)}

#Reconstruct time series' from eigentriplet groups
 r.1<-reconstruct(s,groups=group.1)  #group 1 
 recon.1<-r.1$F1 
 r.2<-reconstruct(s,groups=group.2)  #group 2 
 recon.2<-r.2$F1 
 r.3<-reconstruct(s,groups=group.3)  #group 3 
 recon.3<-r.3$F1 
 
#Signal/Noise
#recon<-reconstruct(s,groups=list(1:5));recon<-r$F1;res<-residuals(r)
signal<-recon.1+recon.2+recon.3
noise<-x-signal  #residuals to composite reconstruction

#Export reconstructed series
data<-cbind(x,signal,noise,recon.1,recon.2,recon.3)
write.table(data,sep=",",col.names=c("data","signal","noise","recon.1",
  "recon.2","recon.3"),"data.csv",row.names=FALSE)
} #end if n.group=3

###################################
#Results for 4 eigentriplet groups#
###################################
if (n.groups==4) {
#Compute periods of eigentriplet groups
if(length(group.1)==2) {
  cycle.1<-parestimate(s,groups=list(group.1))[2]}
if(length(group.2)==2){
  cycle.2<-parestimate(s,groups=list(group.2))[2]}
if(length(group.3)==2) {
  cycle.3<-parestimate(s,groups=list(group.3))[2]}
if(length(group.4)==2){
  cycle.4<-parestimate(s,groups=list(group.4))[2]}
if(length(group.1)==2) {
  cycles<-cbind(cycle.1,cycle.2,cycle.3,cycle.4)} else {
  cycles<-cbind(cycle.2,cycle.3,cycle.4)}

#Reconstruct time series' from eigentriplet groups
r.1<-reconstruct(s,groups=group.1)  #group 1 
recon.1<-r.1$F1 
r.2<-reconstruct(s,groups=group.2)  #group 2 
recon.2<-r.2$F1 
r.3<-reconstruct(s,groups=group.3)  #group 3 
recon.3<-r.3$F1 
r.4<-reconstruct(s,groups=group.4)  #group 4 
recon.4<-r.4$F1 

#Signal/Noise
#recon<-reconstruct(s,groups=list(1:5));recon<-r$F1;res<-residuals(r)
signal<-recon.1+recon.2+recon.3+recon.4
noise<-x-signal  #residuals to composite reconstruction

#Export reconstructed series
data<-cbind(x,signal,noise,recon.1,recon.2,recon.3,recon.4)
write.table(data,sep=",",col.names=c("data","signal","noise","recon.1",
  "recon.2","recon.3","recon.4"),"data.csv",row.names=FALSE)
} #end if n.group=4

###################################
#Results for 5 eigentriplet groups#
###################################
if (n.groups==5) {
#Compute periods of eigentriplet groups
if(length(group.1)==2) {
  cycle.1<-parestimate(s,groups=list(group.1))[2]}
if(length(group.2)==2){
  cycle.2<-parestimate(s,groups=list(group.2))[2]}
if(length(group.3)==2) {
  cycle.3<-parestimate(s,groups=list(group.3))[2]}
if(length(group.4)==2){
  cycle.4<-parestimate(s,groups=list(group.4))[2]}
if(length(group.5)==2){
  cycle.5<-parestimate(s,groups=list(group.5))[2]}
if(length(group.1)==2) {
  cycles<-cbind(cycle.1,cycle.2,cycle.3,cycle.4,cycle.5)} else {
  cycles<-cbind(cycle.2,cycle.3,cycle.4,cycle.5)}

#Reconstruct time series' from eigentriplet groups
r.1<-reconstruct(s,groups=group.1)  #group 1 
recon.1<-r.1$F1 
r.2<-reconstruct(s,groups=group.2)  #group 2 
recon.2<-r.2$F1 
r.3<-reconstruct(s,groups=group.3)  #group 3 
recon.3<-r.3$F1 
r.4<-reconstruct(s,groups=group.4)  #group 4 
recon.4<-r.4$F1 
r.5<-reconstruct(s,groups=group.5)  #group 5 
recon.5<-r.5$F1 

#Signal/Noise
#recon<-reconstruct(s,groups=list(1:5));recon<-r$F1;res<-residuals(r)
signal<-recon.1+recon.2+recon.3+recon.4+recon.5
noise<-x-signal  #residuals to composite reconstruction

#Export reconstructed series
data<-cbind(x,signal,noise,recon.1,recon.2,recon.3,recon.4,recon.5)
write.table(data,sep=",",col.names=c("data","signal","noise","recon.1",
  "recon.2","recon.3","recon.4","recon.5"),"data.csv",row.names=FALSE)
} #end if n.group=5

###################################
#Results for 6 eigentriplet groups#
###################################
if (n.groups==6) {
#Compute periods of eigentriplet groups
if(length(group.1)==2) {
  cycle.1<-parestimate(s,groups=list(group.1))[2]}
if(length(group.2)==2){
  cycle.2<-parestimate(s,groups=list(group.2))[2]}
if(length(group.3)==2) {
  cycle.3<-parestimate(s,groups=list(group.3))[2]}
if(length(group.4)==2){
  cycle.4<-parestimate(s,groups=list(group.4))[2]}
if(length(group.5)==2){
  cycle.5<-parestimate(s,groups=list(group.5))[2]}
if(length(group.6)==2){
  cycle.6<-parestimate(s,groups=list(group.6))[2]}

if(length(group.1)==2) {
  cycles<-cbind(cycle.1,cycle.2,cycle.3,cycle.4,cycle.5,cycle.6)} else {
  cycles<-cbind(cycle.2,cycle.3,cycle.4,cycle.5,cycle.6)}

#Reconstruct time series' from eigentriplet groups
r.1<-reconstruct(s,groups=group.1)  #group 1 
recon.1<-r.1$F1 
r.2<-reconstruct(s,groups=group.2)  #group 2 
recon.2<-r.2$F1 
r.3<-reconstruct(s,groups=group.3)  #group 3 
recon.3<-r.3$F1 
r.4<-reconstruct(s,groups=group.4)  #group 4 
recon.4<-r.4$F1 
r.5<-reconstruct(s,groups=group.5)  #group 5 
recon.5<-r.5$F1 
r.6<-reconstruct(s,groups=group.6)  #group 6 
recon.6<-r.6$F1 

#Signal/Noise
#recon<-reconstruct(s,groups=list(1:5));recon<-r$F1;res<-residuals(r)
signal<-recon.1+recon.2+recon.3+recon.4+recon.5+recon.6
noise<-x-signal  #residuals to composite reconstruction

#Export reconstructed series
data<-cbind(x,signal,noise,recon.1,recon.2,recon.3,recon.4,recon.5,
  recon.6)
write.table(data,sep=",",col.names=c("data","signal","noise","recon.1",
  "recon.2","recon.3","recon.4","recon.5","recon.6"),"data.csv",
  row.names=FALSE)
} #end if n.group=6

#Print periods of cycles:  Note cycles will not be computed if
#group.1 does not have eigentriplet pair.
