embed_udf <-
function(x){
  
  #Step 1: Calculate embedding delay
  
  #Step 1a. Mutual information function
  
  library(tseriesChaos)
  mutual.inf<-mutual(x)  #mutal(tseriesChaos)  
  
  #Step 1b: Find delay at which mutual information function hits its first minimum
  
  mutual.em<-embedd(mutual.inf,2,1)  #embedd(tseriesChaos) 
  adj.length<-length(mutual.inf)-1   #lose 1 observation to lag     
  
  hold<-matrix(0,adj.length,1)
  for (i in 1:adj.length){
    test<-if(mutual.em[i,1]>mutual.em[i,2])TRUE else break
    hold[i,1]<-test  #"TRUE' = 1 in R, so min delay occurs at sum(TRUE) 
  }  #end iloop
  
  out<-sum(hold)
  
  #Step 1c: If the mutual information function is decreasing across all 20 delays, set 
  #delay at max = 20
  
  d<-if(out<20)out else 20  #Embedding delay
  
  #Step 2: Compute Theiler window
  
  #Step 2a: Autocorrelation function
  
  library(FinTS)
  lag.max=50
  out<-Acf(x,lag.max)  #Acf(FinTS) (autocorrelation function)
  
  #Step 2b: Find delay at which Acf is first zero 
  
  hold<-matrix(0,lag.max,1)
  for (i in 1:lag.max){
    out1<-as.character(out[i])
    out2<-out1[1]
    out3<-if(out2>0)TRUE else break
    hold[i,1]<-out3
  }  #end iloop
  
  t<-sum(hold)+1   #Theiler Window 
  
  #Step 3: Embedding dimension
  
  #Step 3a: False nearest neighbors function
  
  library(zoo)  
  out<-false.nearest(x,10,d,t)  #false.nearest(tseriesChaos)
  #'10' is maximum embedding dimension
  #theiler(user-defined)
  
  #Step 3b: Find delay at which false nearest neighbors decrease below set tolerance
  
  out[is.na(out)] <- 0
  out1<-c(out[1],out[3],out[5],out[7],out[9],out[11],out[13],out[15],out[17],out[19])
  out1<-as.matrix(out1)
  out2<-out1>0.15 #set tolerance of 15% 
  out3<-sum(out2)  #sum(zoo)  
  m<-if(out3<10)out3+1 else 10 #Embedding dimension 
  
  #If m=1, program crashes due to 'subscript out of bounds' error--need to
  #guarantee an embedding dimension of at least two:
  
  if(m<=1){m<-2} else {m}  
  
  #Step 4: Embed time series
  
  Mx<-embedd(x,m,d)
  
  #Results
  
  results.embed_udf<-list(d,t,m,Mx) 
  return(results.embed_udf)    
  
}
